package ac.essex.ca.neighbourhoods;

import ac.essex.ca.util.Position;

import java.util.Vector;

/**
 * <p>
 * A neighbourhood defines the pixels around a given pixel
 * which are to be examined. Different shaped neighbourhoods
 * can produce different behaviours.
 * </p>
 *
 * <p>
 * The neighbourhood is defined in terms of relative positions about
 * the central pixel which is at (0,0). Once a neighbourhood is defined
 * a set of rules can be applied to it which either switch the central pixel
 * on or off.
 * </p>
 *
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 08-Aug-2006
 * @version 1.0
 */
public abstract class Neighbourhood {

    public Vector<Position> positions;

    public Neighbourhood() {
        positions = new Vector<Position>(10);
        // call the abstracted method
        createNeighbourhood();
    }

    public void addPosition(int x, int y) {
        // create a new position object
        Position p = new Position(x, y);
        // ensure it isn't added twice
        if (positions.contains(p)) throw new RuntimeException("Bug in your code: You can't add the position at " + x + ", " + y + " to the neighbourhood twice.");
        // add to the vector.
        positions.add(new Position(x, y));
    }

    /**
     * Returns all the positions in the neighbourhood. All positions are saved as relative coordinates
     * to a central pixel.
     */
    public Vector<Position> getPositions() {
        return positions;
    }

    public Position getPosition(int index) {
        return positions.elementAt(index);
    }

    /**
     * Initialises the rest of the positions within the neighbourhood
     * to produce the kind of shape we want. See subclasses of this object
     * for examples.
     */
    public abstract void createNeighbourhood();


}
