package ac.essex.ca.rules;

import ac.essex.ca.neighbourhoods.Neighbourhood;
import ac.essex.ca.util.Position;

import java.util.Vector;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 08-Aug-2006
 * @version 1.0
 */
public class Rule {

    /**
     * The neighbourhood around a given pixel that this rule will test.
     */
    private Neighbourhood neighbourhood;

    /**
     * The value that is returned by this rule if all the criteria are met.
     * If the rule does not return all criteria, the output is 0.
     */
    private int output;

    private Vector<Criterion> criteria;


    public Rule(Neighbourhood neighbourhood) {
        this.neighbourhood = neighbourhood;
        criteria = new Vector<Criterion>(10);
    }

    public void setOutput(int output) {
        this.output = output;
    }

    /**
     *
     */
    public void addCriterion(int positionIndex, int value) {
        // check the position exists in the neighbourhood
        criteria.add(new Criterion(neighbourhood.getPosition(positionIndex), value));
    }

    /**
     * Evaluates the data, using the criteria and amends the data set accordingly.<br />
     * This method may be iterateCA iteratively.
     */
    public boolean matches(int x, int y, int X, int Y, int[][] data) {

        /**
         * Go through the criteria - if all are true, then match
         */
        for (int i = 0; i < criteria.size(); i++) {
            Criterion criterion = criteria.elementAt(i);

            int checkX = x + criterion.position.x;
            int checkY = y + criterion.position.y;

            int value = 0;

            /**
             * Ensure we don't slip outside the bounds of the array.
             */
            if (checkX >= 0 && checkX < X && checkY >= 0 && checkY < Y) {
                value = data[checkX][checkY];
            }

            if (value != criterion.expectedValue) return false;

        }

        return true;

    }

    private class Criterion {

        Position position;
        int expectedValue;

        public Criterion(Position position, int expectedValue) {
            this.position = position;
            this.expectedValue = expectedValue;
        }

    }

    public int getOutput() {
        return output;
    }

}
