package ac.essex.ca.rules;

import ac.essex.ca.neighbourhoods.Neighbourhood;
import ac.essex.ca.neighbourhoods.VonNeumannNeighbourhood;

import java.util.Vector;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 08-Aug-2006
 * @version 1.0
 */
public class RuleSet {

    Vector<Rule> rules;

    public RuleSet() {

        rules = new Vector<Rule>(10);

    }

    public RuleSet(int ruleNumber)  {

        this();

        // Create a neighbourhood
        Neighbourhood n = new VonNeumannNeighbourhood();

        /**
         * Get the binarised version of this number
         */
        String binstr = Integer.toBinaryString(ruleNumber);

        /**
         * Pad out the binary to make sure it is 8 bit.
         */
        while (binstr.length() < 8) {
            binstr = "0" + binstr;
        }


        /**
         * Creates a rule set for the neighbourhood.
         */
        Rule rule8 = new Rule(n);
        rule8.addCriterion(0, 1);
        rule8.addCriterion(1, 1);
        rule8.addCriterion(2, 1);

        Rule rule7 = new Rule(n);
        rule7.addCriterion(0, 1);
        rule7.addCriterion(1, 1);
        rule7.addCriterion(2, 0);

        Rule rule6 = new Rule(n);
        rule6.addCriterion(0, 1);
        rule6.addCriterion(1, 0);
        rule6.addCriterion(2, 1);

        Rule rule5 = new Rule(n);
        rule5.addCriterion(0, 1);
        rule5.addCriterion(1, 0);
        rule5.addCriterion(2, 0);

        Rule rule4 = new Rule(n);
        rule4.addCriterion(0, 0);
        rule4.addCriterion(1, 1);
        rule4.addCriterion(2, 1);

        Rule rule3 = new Rule(n);
        rule3.addCriterion(0, 0);
        rule3.addCriterion(1, 1);
        rule3.addCriterion(2, 0);

        Rule rule2 = new Rule(n);
        rule2.addCriterion(0, 0);
        rule2.addCriterion(1, 0);
        rule2.addCriterion(2, 1);

        Rule rule1 = new Rule(n);
        rule1.addCriterion(0, 0);
        rule1.addCriterion(1, 0);
        rule1.addCriterion(2, 0);

        /**
         * Add the rules to this set
         */
        addRule(rule8);
        addRule(rule7);
        addRule(rule6);
        addRule(rule5);
        addRule(rule4);
        addRule(rule3);
        addRule(rule2);
        addRule(rule1);

        System.out.println(binstr);

        /**
         * Set the outputs of the rules according to the decoded number
         */
        rule8.setOutput(Integer.parseInt(binstr.substring(0, 1)));
        rule7.setOutput(Integer.parseInt(binstr.substring(1, 2)));
        rule6.setOutput(Integer.parseInt(binstr.substring(2, 3)));
        rule5.setOutput(Integer.parseInt(binstr.substring(3, 4)));
        rule4.setOutput(Integer.parseInt(binstr.substring(4, 5)));
        rule3.setOutput(Integer.parseInt(binstr.substring(5, 6)));
        rule2.setOutput(Integer.parseInt(binstr.substring(6, 7)));
        rule1.setOutput(Integer.parseInt(binstr.substring(7, 8)));


    }

    public void addRule(Rule r) {
        rules.add(r);
    }

    public int[][] evaluate(int X, int Y, int[][] data) {

        // construct a new data array into which the new cell's values are added
        int[][] newData = new int[X][Y];

        for (int y = 0; y < Y; y++) {
            for (int x = 0; x < X; x++) {

                // for each value, find the rule which matches this cell configuration
                for (int j = 0; j < rules.size(); j++) {
                    Rule rule = rules.elementAt(j);
                    if (rule.matches(x,y,X,Y,data)) {
                        if (rule.getOutput() > 0) {
                            data[x][y] = rule.getOutput();
                        }
                        break;
                    }
                }

            }
        }

        return data;


    }




}
